package com.wee0.flutter.bluetooth_helper;

import android.app.Activity;
import android.app.Application;
import android.bluetooth.BluetoothManager;
import android.content.Context;

import androidx.annotation.NonNull;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * BluetoothHelperPlugin
 */
public class BluetoothHelperPlugin implements FlutterPlugin, ActivityAware, MethodCallHandler, PluginRegistry.RequestPermissionsResultListener {

    private static final BluetoothHelperPlugin _me = new BluetoothHelperPlugin();

    // 插件绑定对象
    private static Registrar _registrar;
    private static FlutterPluginBinding _flutterPluginBinding;
    private static ActivityPluginBinding _activityPluginBinding;
    private static BinaryMessenger _messenger;

    // 交互通道
    private MethodChannel _methodChannel;
    private EventChannel _eventChannel;

    // 初始化操作
    private synchronized void _init() {
        MyLog.debug("init begin...");
        MyLog.debug("version: " + android.os.Build.VERSION.SDK_INT);
        MyLog.debug("version > 18 ?: " + (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.JELLY_BEAN_MR2));
        MyLog.debug("version > 21 ?: " + (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.LOLLIPOP));
        MyLog.debug("_registrar: " + _registrar);
        MyLog.debug("_flutterPluginBinding: " + _flutterPluginBinding);
        MyLog.debug("_activityPluginBinding: " + _activityPluginBinding);
        MyLog.debug("_activity: " + PlatformHelper.me().getActivity());
        MyLog.debug("_application: " + PlatformHelper.me().getApplication());
        MyLog.debug("_messenger: " + _me._messenger);

        if (null != _me._methodChannel) {
            MyLog.debug("skip repetition init.");
            return;
        }

        Activity _activity = null;
        Application _application = null;
        if (null != _registrar) {
            _registrar.addRequestPermissionsResultListener(_me);
            _activity = _registrar.activity();
            if (null != _activity) {
                _application = _activity.getApplication();
            } else if (null != _registrar.context()) {
                _application = (Application) _registrar.context().getApplicationContext();
            }
            _messenger = _registrar.messenger();
        } else {
            if (null != _activityPluginBinding) {
                _activityPluginBinding.addRequestPermissionsResultListener(_me);
                _activity = _activityPluginBinding.getActivity();
                _application = null == _activity ? (Application) _flutterPluginBinding.getApplicationContext() : _activity.getApplication();
                _messenger = _flutterPluginBinding.getBinaryMessenger();
//                _messenger = _flutterPluginBinding.getFlutterEngine().getDartExecutor();
            }
        }
        PlatformHelper.me().setActivity(_activity);
        PlatformHelper.me().setApplication(_application);

        MyLog.debug("_activity1: " + PlatformHelper.me().getActivity());
        MyLog.debug("_application1: " + PlatformHelper.me().getApplication());
        MyLog.debug("_messenger1: " + _messenger);

        if (null == _messenger) {
            MyLog.debug("BinaryMessenger cant not be null!");
            return;
        }
        MyLog.debug("int methodChannel to BinaryMessenger: " + _messenger);
        _me._methodChannel = new MethodChannel(_messenger, "bluetooth_helper");
        _me._methodChannel.setMethodCallHandler(_me);

        BluetoothManager _bluetoothManager = (BluetoothManager) _application.getSystemService(Context.BLUETOOTH_SERVICE);
        MyBluetoothManager.me().init(_bluetoothManager);
        MyLog.debug("bluetooth.isEnabled: " + MyBluetoothManager.me().isEnabled());

        MyLog.debug("init end.");
    }

    // 资源销毁操作
    private synchronized void _destroy() {
        MyLog.debug("destroy begin...");
        if (null != _me._methodChannel) {
            _me._methodChannel.setMethodCallHandler(null);
            _me._methodChannel = null;
        }
        MyBluetoothManager.me().destroy();
        PlatformHelper.me().destroy();

        if (null != _messenger) _messenger = null;
        if (null != _registrar) _registrar = null;
        if (null != _flutterPluginBinding) _flutterPluginBinding = null;
        if (null != _activityPluginBinding) {
            _activityPluginBinding.removeRequestPermissionsResultListener(_me);
            _activityPluginBinding = null;
        }
        if (null != _registrar) _registrar = null;

        MyLog.debug("destroy end.");
    }

    public static void registerWith(Registrar registrar) {
        MyLog.debug("registerWith: " + registrar);
        _registrar = registrar;
        _me._init();
    }

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        MyLog.debug("onAttachedToEngine: " + flutterPluginBinding);
        _flutterPluginBinding = flutterPluginBinding;
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        MyLog.debug("onDetachedFromEngine: " + binding);
        _flutterPluginBinding = null;
    }

    @Override
    public void onAttachedToActivity(ActivityPluginBinding binding) {
        MyLog.debug("onAttachedToActivity: " + binding);
        _activityPluginBinding = binding;
        _me._init();
    }

    @Override
    public void onDetachedFromActivity() {
        MyLog.debug("onDetachedFromActivity...");
        _me._destroy();
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {
        MyLog.debug("onDetachedFromActivityForConfigChanges...");
        onDetachedFromActivity();
    }

    @Override
    public void onReattachedToActivityForConfigChanges(ActivityPluginBinding binding) {
        MyLog.debug("onReattachedToActivityForConfigChanges: " + binding);
        onAttachedToActivity(binding);
    }


    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        MyLog.debug("onMethodCall: " + call);
        switch (call.method) {
            case "getPlatformVersion":
                result.success("Android " + android.os.Build.VERSION.RELEASE);
                break;
            case "scan":
                MyBluetoothManager.me().scan();
                result.success(null);
                break;
            case "connect":
                MyLog.debug("connect arguments:" + call.arguments);
                String _deviceId = (String) call.arguments;
                if (null == _deviceId) _deviceId = "D8:CB:BD:91:D5:D5";
                MyBluetoothManager.me().connect(_deviceId, false);
                result.success(null);
                break;
            case "disconnect":
                MyLog.debug("disconnect arguments:" + call.arguments);
                String _deviceId1 = (String) call.arguments;
                if (null == _deviceId1) _deviceId1 = "D8:CB:BD:91:D5:D5";
                MyBluetoothManager.me().disconnect(_deviceId1);
                result.success(null);
                break;
            case "check":
//                PermissionHelper.me().test(_activity);
                result.success("check result: " + MyLocationManager.me.getLocation());
                break;
            case "test":
                result.success(String.valueOf(MyBluetoothManager.me().isEnabled()));
                break;
            default:
                result.notImplemented();
        }
    }

    @Override
    public boolean onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
        try {
            return PermissionHelper.me().onRequestPermissionsResult(requestCode, permissions, grantResults);
        } catch (Exception e) {
//            result.error("no_permissions", e.getMessage(), null);
            return false;
        }
    }

    static void onReceiveData(final byte[] data) {
        PlatformHelper.me().getActivity().runOnUiThread(new Runnable() {
            @Override
            public void run() {
                _me._methodChannel.invokeMethod("onReceiveData", data);
            }
        });
    }

}
