package com.wee0.flutter.bluetooth_helper;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattDescriptor;
import android.bluetooth.BluetoothGattService;
import android.bluetooth.BluetoothManager;
import android.bluetooth.BluetoothProfile;
import android.bluetooth.le.BluetoothLeScanner;
import android.bluetooth.le.ScanCallback;
import android.bluetooth.le.ScanFilter;
import android.bluetooth.le.ScanResult;
import android.bluetooth.le.ScanSettings;
import android.content.Context;
import android.location.Address;
import android.location.Criteria;
import android.location.Geocoder;
import android.location.Location;
import android.location.LocationManager;
import android.os.Handler;
import android.provider.Settings;

import java.io.IOException;
import java.io.ObjectStreamException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

/**
 * 自定义蓝牙管理
 */
final class MyBluetoothManager {

    // 托管的蓝牙管理器对象
    private BluetoothManager _bluetoothManager;
    private BluetoothAdapter _bluetoothAdapter;
    private Handler _handler;
    private ScanCallback _scanCallback;
    private Map<String, BluetoothGatt> _data;
    private final BluetoothGattCallback _gattCallback = new BluetoothGattCallback() {
        @Override
        public void onConnectionStateChange(BluetoothGatt gatt, int status, int newState) {
            MyLog.debug("onConnectionStateChange gatt:" + gatt + " status:" + status + ", newState:" + newState);
            MyLog.debug("gatt: " + gatt);
            MyLog.debug("data:" + _data);
            if (BluetoothProfile.STATE_CONNECTED == newState) {
                MyLog.debug("connected gatt:" + gatt);
                gatt.discoverServices();
            } else if (BluetoothProfile.STATE_DISCONNECTED == newState) {
                //:TODO 通知应用连接断开事件
                MyLog.debug("disconnected gatt:" + gatt);
                gatt.close();
            }
            super.onConnectionStateChange(gatt, status, newState);
        }

        @Override
        public void onServicesDiscovered(BluetoothGatt gatt, int status) {
            MyLog.debug("onServicesDiscovered gatt:" + gatt + " status:" + status);
            if (BluetoothGatt.GATT_SUCCESS == status) {
                BluetoothGattCharacteristic _readCharacteristic = null;
                BluetoothGattCharacteristic _writeCharacteristic = null;
                List<BluetoothGattService> _gattServices = gatt.getServices();
                for (BluetoothGattService _gattService : _gattServices) {
                    MyLog.print(_gattService);
                    List<BluetoothGattCharacteristic> _characteristics = _gattService.getCharacteristics();
                    for (BluetoothGattCharacteristic _characteristic : _characteristics) {
                        MyLog.print(_characteristic);
                        String _uuid = _characteristic.getUuid().toString();
                        if (0 == _uuid.indexOf("00001526-")) {
                            MyLog.debug("change read characteristic: " + _uuid);
                            _readCharacteristic = _characteristic;
                        } else if (0 == _uuid.indexOf("00001525-")) {
                            MyLog.debug("change write characteristic: " + _uuid);
                            _writeCharacteristic = _characteristic;
                        }
                    }
                }
                if (null != _readCharacteristic) {
                    boolean _setNotificationResult = gatt.setCharacteristicNotification(_readCharacteristic, true);
                    MyLog.debug("setNotificationResult:" + _setNotificationResult);

                    BluetoothGattDescriptor _gattDescriptor = _readCharacteristic.getDescriptor(BluetoothConstants.descCharacteristicClientConfig);
                    if (null != _gattDescriptor) {
                        _gattDescriptor.setValue(BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE);
                        gatt.writeDescriptor(_gattDescriptor);
                    }
                }
                if (null != _writeCharacteristic) {

                }
            }

        }

        @Override
        public void onCharacteristicChanged(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic) {
            MyLog.debug("onCharacteristicChanged gatt:" + gatt + ", characteristic:" + characteristic);
            MyLog.print(characteristic);
            BluetoothHelperPlugin.onReceiveData(characteristic.getValue());
        }

        @Override
        public void onCharacteristicRead(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
            MyLog.debug("onCharacteristicRead gatt:" + gatt + ", characteristic:" + characteristic + ", status:" + status);
            super.onCharacteristicWrite(gatt, characteristic, status);
            if (BluetoothGatt.GATT_SUCCESS == status) {
                MyLog.debug("read:" + characteristic.getValue());
            }
        }

        @Override
        public void onCharacteristicWrite(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
            MyLog.debug("onCharacteristicWrite gatt:" + gatt + ", characteristic:" + characteristic + ", status:" + status);
            super.onCharacteristicWrite(gatt, characteristic, status);
            if (BluetoothGatt.GATT_SUCCESS == status) {
                MyLog.debug("write:" + characteristic.getValue());
            }
        }

        @Override
        public void onDescriptorWrite(BluetoothGatt gatt, BluetoothGattDescriptor descriptor, int status) {
            MyLog.debug("onDescriptorWrite gatt:" + gatt + ", characteristic:" + descriptor + ", status:" + status);
            super.onDescriptorWrite(gatt, descriptor, status);
        }
    };

    /**
     * 初始化逻辑
     *
     * @param bluetoothManager 托管的蓝牙管理器对象
     */
    void init(BluetoothManager bluetoothManager) {
        if (null == bluetoothManager) throw new IllegalStateException("bluetoothManager can not be null!");
        this._bluetoothManager = bluetoothManager;
        this._bluetoothAdapter = _bluetoothManager.getAdapter();
        MyLog.debug("MyBluetoothManager.init...");
        MyLog.debug("MyBluetoothManager.bluetoothManager: " + this._bluetoothManager);
        MyLog.debug("MyBluetoothManager.bluetoothAdapter: " + this._bluetoothAdapter);
//        Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
//        activity.startActivityForResult(enableBtIntent, 1);
        this._handler = new Handler();
        this._data = new HashMap<>(8);
    }

    /**
     * 资源释放逻辑
     */
    void destroy() {
        this._data.clear();
        this._data = null;
        this._bluetoothAdapter = null;
        this._bluetoothManager = null;
        this._handler = null;
        this._scanCallback = null;
    }

    /**
     * @return 蓝牙是否开启
     */
    public boolean isEnabled() {
        return null != this._bluetoothAdapter && this._bluetoothAdapter.isEnabled();
    }

    public void connect(String address, boolean autoConnect) {
        if (!isEnabled()) throw new IllegalStateException("bluetooth is disabled.");
        if (!MyLocationManager.me.isEnabled()) throw new IllegalStateException("location is disabled.");

        BluetoothDevice _device = this._bluetoothAdapter.getRemoteDevice(address);
        MyLog.debug("device:" + _device);
        boolean _isConnected = this._bluetoothManager.getConnectedDevices(BluetoothProfile.GATT).contains(_device);
        MyLog.debug("device:" + _device + ", isConnected:" + _isConnected);
        if (_isConnected) {
            MyLog.debug("device " + _device + " already isConnected.");
            return;
        }
        BluetoothGatt _gatt = null;
        if (PlatformHelper.sdkGE23()) {
            _gatt = _device.connectGatt(PlatformHelper.me().getActivity(), autoConnect, _gattCallback, BluetoothDevice.TRANSPORT_LE);
        } else {
            _gatt = _device.connectGatt(PlatformHelper.me().getActivity(), autoConnect, _gattCallback);
        }
        MyLog.debug("address:" + address + ", gatt:" + _gatt);
        this._data.put(address, _gatt);
    }

    public void disconnect(String address) {
        // "D8:CB:BD:91:D5:D5"
        BluetoothDevice _device = this._bluetoothAdapter.getRemoteDevice(address);
        int _state = this._bluetoothManager.getConnectionState(_device, BluetoothProfile.GATT);
        MyLog.debug("disconnect state:" + _state);
        BluetoothGatt _gatt = this._data.get(address);
        if (null != _gatt) {
            MyLog.debug("disconnect _gatt:" + _gatt);
            _gatt.disconnect();
            if (BluetoothProfile.STATE_DISCONNECTED == _state) {
                MyLog.debug("close _gatt:" + _gatt);
                _gatt.close();
            }
        }
    }

    // 默认的扫描时间为3秒。
    private static final long _DEF_SCAN_PERIOD = 3000;

    public void scan() {
        if (null != this._scanCallback) {
            MyLog.debug("skip scanning...");
            return;
        }
        this._scanCallback = new ScanCallback() {
            @Override
            public void onScanResult(int callbackType, ScanResult result) {
                MyLog.debug("onScanResult. callbackType: " + callbackType + ", result: " + result);
            }

            @Override
            public void onBatchScanResults(List<ScanResult> results) {
                MyLog.debug("onBatchScanResults. results: " + results);
            }

            @Override
            public void onScanFailed(int errorCode) {
                MyLog.debug("onScanFailed. errorCode: " + errorCode);
            }
        };
        MyLog.debug("start scan...");

        final BluetoothLeScanner _scanner = this._bluetoothAdapter.getBluetoothLeScanner();
        if (null == _scanner) throw new IllegalStateException("BluetoothLeScanner can not be null!");
        List<ScanFilter> _scanFilters = new ArrayList<>();
//        ScanFilter.Builder _filterBuilder = new ScanFilter.Builder();
//        _filterBuilder.setDeviceName("");
//        ScanFilter _scanFilter = _filterBuilder.build();
//        _scanFilters.add(_scanFilter);

        ScanSettings.Builder _settingsBuilder = new ScanSettings.Builder();
        _settingsBuilder.setScanMode(ScanSettings.SCAN_MODE_BALANCED);
        ScanSettings _scanSettings = _settingsBuilder.build();
        _scanner.startScan(_scanFilters, _scanSettings, this._scanCallback);
        this._handler.postDelayed(new Runnable() {
            @Override
            public void run() {
                MyLog.debug("before stop scan.");
                _scanner.stopScan(_scanCallback);
                _scanCallback = null;
                MyLog.debug("after stop scan.");
            }
        }, _DEF_SCAN_PERIOD);
    }

    /************************************************************
     ************* 单例对象。
     ************************************************************/
    private MyBluetoothManager() {
        if (null != MyBluetoothManagerHolder._INSTANCE) {
            // 防止使用反射API创建对象实例。
            throw new IllegalStateException("that's not allowed!");
        }
    }

    // 当前对象唯一实例持有者。
    private static final class MyBluetoothManagerHolder {
        private static final MyBluetoothManager _INSTANCE = new MyBluetoothManager();
    }

    // 防止使用反序列化操作获取多个对象实例。
    private Object readResolve() throws ObjectStreamException {
        return MyBluetoothManagerHolder._INSTANCE;
    }

    /**
     * 获取当前对象唯一实例。
     *
     * @return 当前对象唯一实例
     */
    public static MyBluetoothManager me() {
        return MyBluetoothManagerHolder._INSTANCE;
    }

}
