package com.wee0.flutter.bluetooth_helper;

import android.Manifest;
import android.app.Activity;
import android.content.pm.PackageManager;

import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import java.io.ObjectStreamException;
import java.util.Arrays;

/**
 * 权限操作助手
 */
final class PermissionHelper {

    private static final int _REQUEST_FINE_LOCATION_PERMISSIONS = 1452;

    public boolean checkPermission() {
        return true;
    }

    /**
     * 权限请求响应结果处理
     *
     * @param requestCode
     * @param permissions
     * @param grantResults
     * @return
     */
    public boolean onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
        if (requestCode == _REQUEST_FINE_LOCATION_PERMISSIONS) {
            MyLog.debug("permissions: " + Arrays.toString(permissions));
            MyLog.debug("grantResults: " + Arrays.toString(grantResults));
            if (grantResults[0] != PackageManager.PERMISSION_GRANTED) {
                throw new RuntimeException("bluetooth_helper plugin requires location permissions for scanning");
            }
//            startScan(pendingCall, pendingResult);
            return true;
        }
        return false;
    }

    void test(Activity activity) {
        Boolean _has = activity.getPackageManager().hasSystemFeature(PackageManager.FEATURE_BLUETOOTH_LE);
        MyLog.debug("has FEATURE_BLUETOOTH_LE: " + _has);

        int _check = ContextCompat.checkSelfPermission(activity, Manifest.permission.ACCESS_FINE_LOCATION);
        if (PackageManager.PERMISSION_GRANTED == _check) return;
        ActivityCompat.requestPermissions(
                activity,
                new String[]{
                        Manifest.permission.ACCESS_FINE_LOCATION
                },
                _REQUEST_FINE_LOCATION_PERMISSIONS);
    }

    /************************************************************
     ************* 单例对象。
     ************************************************************/
    private PermissionHelper() {
        if (null != PermissionHelperHolder._INSTANCE) {
            // 防止使用反射API创建对象实例。
            throw new IllegalStateException("that's not allowed!");
        }
    }

    // 当前对象唯一实例持有者。
    private static final class PermissionHelperHolder {
        private static final PermissionHelper _INSTANCE = new PermissionHelper();
    }

    // 防止使用反序列化操作获取多个对象实例。
    private Object readResolve() throws ObjectStreamException {
        return PermissionHelperHolder._INSTANCE;
    }

    /**
     * 获取当前对象唯一实例。
     *
     * @return 当前对象唯一实例
     */
    public static PermissionHelper me() {
        return PermissionHelperHolder._INSTANCE;
    }
}
