import 'package:flutter/material.dart';
import 'dart:async';

import 'package:flutter/services.dart';
import 'package:bluetooth_helper/bluetooth_helper.dart';

void main() => runApp(MyApp());

class MyApp extends StatefulWidget {
  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  String _platformVersion = 'Unknown';
  String _deviceId = "C3:4B:F0:6B:77:1D";

  @override
  void initState() {
    super.initState();
    initPlatformState();
  }

  // Platform messages are asynchronous, so we initialize in an async method.
  Future<void> initPlatformState() async {
    String platformVersion;
    // Platform messages may fail, so we use a try/catch PlatformException.
    try {
      platformVersion = await BluetoothHelper.me.platformVersion;
    } on PlatformException {
      platformVersion = 'Failed to get platform version.';
    }

    // If the widget was removed from the tree while the asynchronous platform
    // message was in flight, we want to discard the reply rather than calling
    // setState to update our non-existent appearance.
    if (!mounted) return;

    setState(() {
      _platformVersion = platformVersion;
    });
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          title: const Text('Plugin example app'),
        ),
        body: Column(
          children: <Widget>[
            Text('Running on: $_platformVersion\n'),
            Row(
              children: <Widget>[
                RaisedButton(
                  child: Text("check"),
                  onPressed: () => BluetoothHelper.me.check.then((_res) => print("check result: $_res")),
                ),
              ],
            ),
            Row(
              children: <Widget>[
                RaisedButton(
                  child: Text("scan"),
                  onPressed: () => BluetoothHelper.me.scan().then((_res) => print("scan result: $_res")),
                ),
                RaisedButton(
                  child: Text("connect"),
                  onPressed: () => BluetoothHelper.me.connect(_deviceId).then((_res) => print("connect result: $_res")),
                ),
                RaisedButton(
                  child: Text("disconnect"),
                  onPressed: () => BluetoothHelper.me.disconnect(_deviceId).then((_res) => print("disconnect result: $_res")),
                )
              ],
            ),
          ],
        ),
      ),
    );
  }
}
